@@ -30,7 +30,7 @@ class LEDControlWidget(QtWidgets.QWidget, Ui_LEDControlWidget):
         self.slider.setValue(value)
         self.setting_value = False
 
-    def value_changed(self, junk=None):
+    def value_changed(self, _junk=None):
         if not self.setting_value:
             value = self.slider.value()
             self.updater.update(value)
