@@ -2,6 +2,7 @@ import bisect
 import datetime
 import functools
 import logging
+import math
 import os
 import subprocess
 import sys
@@ -9,11 +10,15 @@ import tempfile
 import threading
 import urllib.parse
 
-from PySide2 import QtCore, QtGui, QtWidgets
+import diskcache
+from PySide2 import QtCore, QtGui, QtSvg, QtWidgets
 
 import asphodel
+import hyperborea.alert
 import hyperborea.download
 from hyperborea.preferences import read_bool_setting
+from hyperborea.preferences import read_int_setting
+from hyperborea.preferences import write_bool_setting
 import hyperborea.upload
 
 from .ui_plotmain import Ui_PlotMainWindow
@@ -21,7 +26,7 @@ from .about import AboutDialog
 from .device_tab import DeviceTab
 from .connect_tcp_dialog import ConnectTCPDialog
 from .download_firmware_dialog import DownloadFirmwareDialog
-from .preferences import PreferencesDialog
+from .preferences import PreferencesDialog, set_style
 from .tcp_scan_dialog import TCPScanDialog
 
 logger = logging.getLogger(__name__)
@@ -58,18 +63,22 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
     def __init__(self, proxy_manager, parent=None):
         super().__init__(parent)
 
-        self.is_paused = False
         self.disable_streaming = False
         self.disable_archiving = False
-        self.upload_manager = None  # will be created later, if necessary
+
+        # these will be created later, if necessary
+        self.upload_manager = None
+        self.alert_manager = None
 
         self.rate_update_interval = 0.5
         self.rate_average_period = 5.0
         self.rate_average_period_ms = int(self.rate_average_period * 1000)
 
         self.settings = QtCore.QSettings()
+        self.update_style()
 
         self.load_base_dir()
+        self.create_diskcache()
 
         self.proxy_manager = proxy_manager
         self.proxy_lock = threading.Lock()  # locks access to self.proxies
@@ -83,6 +92,7 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
 
         self.tab_widgets = []  # list of tab widgets, in display order
         self.tab_sort_order = []  # matched to self.tab_widgets
+        self.shown_tab = None
 
         self.rf_power_panels = {}  # key=panel, value=enabled boolean
 
@@ -90,17 +100,23 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
 
         self.setupUi(self)
         self.extra_ui_setup()
+        self.setup_logo()
 
         self.setup_callbacks()
         self.setup_update_actions()
 
         self.create_upload_manager()
+        self.create_alert_manager()
+
+        self.collapsed = read_bool_setting(self.settings, "Collapsed", False)
 
         if sys.platform == "darwin":
             # I couldn't figure out how to make menu work natively
             self.menubar.setNativeMenuBar(False)
 
-        # TODO: restore window geometry
+        # restore window geometry
+        geometry = self.settings.value('Geometry', b'')
+        self.restoreGeometry(geometry)
 
         # schedule the initial connect for the beginning of the main loop
         QtCore.QTimer.singleShot(0, self.initial_device_connect)
@@ -113,6 +129,13 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
             app_name = QtWidgets.QApplication.applicationName()
             self.base_dir = os.path.join(documents_path, app_name + " Data")
 
+    def create_diskcache(self):
+        self.diskcache_dir = self.settings.value("DiskCachePath")
+        if not self.diskcache_dir:
+            self.diskcache_dir = QtCore.QStandardPaths.writableLocation(
+                QtCore.QStandardPaths.CacheLocation)
+        self.diskcache = diskcache.Cache(self.diskcache_dir, size_limit=100e6)
+
     def create_upload_manager(self):
         if self.upload_manager is not None:
             # remove the old one
@@ -139,7 +162,7 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
                     rate_update_interval=self.rate_update_interval,
                     rate_average_period=self.rate_average_period,
                     **upload_options)
-            except:
+            except Exception:
                 msg = "Error starting uploader. Check upload configuration."
                 logger.exception(msg)
                 QtWidgets.QMessageBox.critical(self, self.tr("Error"),
@@ -162,11 +185,75 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
             self.uploadProgress.setVisible(False)
             self.uploadNameLabel.setVisible(False)
 
+    def create_alert_manager(self):
+        if self.alert_manager is not None:
+            # remove the old one
+            self.alert_manager.stop()
+            self.alert_manager = None
+
+        self.settings.beginGroup("AlertEmail")
+        email_enabled = read_bool_setting(self.settings, "Enabled", False)
+
+        email_options = {
+            'from_address': self.settings.value("FromAddress"),
+            'to_address': self.settings.value("ToAddress"),
+            'smtp_host': self.settings.value("SMTPHost"),
+            'smtp_port': read_int_setting(self.settings, "SMTPPort", 587),
+            'use_auth': read_bool_setting(self.settings, "UseAuth", True),
+            'smtp_user': self.settings.value("SMTPUser"),
+            'smtp_password': self.settings.value("SMTPPassword")}
+
+        security = self.settings.value("Security")
+        if security:
+            security = security.strip()
+        if security == "STARTTLS":
+            email_options['security'] = 'starttls'
+        elif security == "SSL":
+            email_options['security'] = 'ssl'
+        elif security == "None":
+            email_options['security'] = ''
+        else:
+            email_options['security'] = 'starttls'
+        self.settings.endGroup()
+
+        if email_enabled:
+            try:
+                self.alert_manager = hyperborea.alert.AlertManager(
+                    email_options)
+            except Exception:
+                msg = "Error with email configuration!"
+                logger.exception(msg)
+                QtWidgets.QMessageBox.critical(self, self.tr("Error"),
+                                               self.tr(msg))
+
+    def setup_logo(self):
+        self.stackedWidget.setCurrentIndex(1)
+        self.logo = QtSvg.QSvgWidget(":/logo.svg")
+        size_policy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored,
+                                            QtWidgets.QSizePolicy.Ignored)
+        self.logo.setSizePolicy(size_policy)
+
+        def resizeEvent(event):
+            size = event.size()
+            margins = self.logoLayout.contentsMargins()
+
+            if size.width() == size.height():
+                return
+
+            full_width = size.width() + margins.left() + margins.right()
+            full_height = size.height() + margins.top() + margins.bottom()
+
+            d = min(full_width, full_height)
+            top = math.ceil((full_height - d) / 2)
+            bottom = math.floor((full_height - d) / 2)
+            left = math.ceil((full_width - d) / 2)
+            right = math.floor((full_width - d) / 2)
+            self.logoLayout.setContentsMargins(left, top, right, bottom)
+        self.logo.resizeEvent = resizeEvent
+
+        self.logoLayout.addWidget(self.logo)
+
     def extra_ui_setup(self):
-        self.actionGroup = QtWidgets.QActionGroup(self)
-        self.actionGroup.addAction(self.actionPause)
-        self.actionGroup.addAction(self.actionResume)
-        self.actionResume.setChecked(True)
         self.warningLabel.setVisible(False)
 
         self.actionEnableRFPower.setEnabled(False)
@@ -210,10 +297,8 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
         self.actionFindTCPDevices.setIcon(QtGui.QIcon.fromTheme("plug_lan"))
         self.actionMarkDirectory.setIcon(QtGui.QIcon.fromTheme("folder_up"))
         self.actionMarkFiles.setIcon(QtGui.QIcon.fromTheme("document_up"))
-        self.actionPause.setIcon(QtGui.QIcon.fromTheme("media_pause"))
         self.actionPreferences.setIcon(QtGui.QIcon.fromTheme("preferences"))
         self.actionRescanUSB.setIcon(QtGui.QIcon.fromTheme("plug_usb"))
-        self.actionResume.setIcon(QtGui.QIcon.fromTheme("media_play_green"))
         self.actionUpdateCurrentBranch.setIcon(QtGui.QIcon.fromTheme(
             "branch_element_new"))
         self.actionUpdateLatestStable.setIcon(QtGui.QIcon.fromTheme("branch"))
@@ -225,8 +310,6 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
             "cloud_computing_download"))
 
     def setup_callbacks(self):
-        self.actionPause.triggered.connect(self.pause)
-        self.actionResume.triggered.connect(self.resume)
         self.actionEnableRFPower.triggered.connect(self.enable_rf_power_cb)
         self.actionDisableRFPower.triggered.connect(self.disable_rf_power_cb)
 
@@ -238,6 +321,8 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
         self.actionDisableArchiving.triggered.connect(
             self.set_disable_archiving)
         self.actionAbout.triggered.connect(self.show_about)
+        self.actionChannelTable.triggered.connect(self.show_channel_table)
+        self.actionShowLogs.triggered.connect(self.show_log_dir)
         self.actionPreferences.triggered.connect(self.show_preferences)
         self.actionClosableTabs.triggered.connect(self.closable_tabs_cb)
 
@@ -257,6 +342,7 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
         self.destroyed.connect(self.destroyed_cb)
 
         self.tabWidget.tabCloseRequested.connect(self.tab_close_requested)
+        self.tabWidget.currentChanged.connect(self.current_tab_changed_cb)
 
         self.actionDownloadFirmware.triggered.connect(
             self.download_firmware)
@@ -315,7 +401,7 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
                     self.commit_hash = lines[1].strip()
                     self.build_key = lines[2].strip()
                     valid_info = True
-            except:
+            except Exception:
                 logger.exception('Could not read build_info.txt')
 
         if not valid_info:
@@ -416,9 +502,11 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
         self.update_progress.reset()
         file.close()
 
-        # stop the upload manager
+        # stop the managers
         if self.upload_manager:
             self.upload_manager.stop()
+        if self.alert_manager:
+            self.alert_manager.stop()
 
         # close all device tabs
         while True:
@@ -565,10 +653,12 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
 
     def create_widget(self, serial_number, sort_location):
         # create a new device tab
-        widget = DeviceTab(self.is_paused, serial_number, self.base_dir,
+        widget = DeviceTab(serial_number, self.base_dir, self.diskcache,
                            self, disable_streaming=self.disable_streaming,
                            disable_archiving=self.disable_archiving,
-                           upload_manager=self.upload_manager)
+                           upload_manager=self.upload_manager,
+                           alert_manager=self.alert_manager,
+                           collapsed=self.collapsed)
         self.device_tabs[serial_number] = widget
 
         # find the insertion index
@@ -578,11 +668,13 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
         self.tabWidget.insertTab(index, widget, self.tr("Connecting..."))
         if index == 0:
             self.tabWidget.setCurrentIndex(index)
+        self.stackedWidget.setCurrentIndex(0)
 
         update_func = functools.partial(self.update_widget_text, widget)
         widget.name_update.connect(update_func)
         close_func = functools.partial(self.close_tab, widget)
         widget.close_pressed.connect(close_func)
+        widget.collapsed_set.connect(self.collapsed_set)
 
     def create_proxy(self, proxy_args, location_string, serial_number,
                      sort_location, reconnect_info=None):
@@ -681,7 +773,7 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
         def rescan():
             try:
                 return asphodel.find_tcp_devices()
-            except:
+            except Exception:
                 return []
 
         dialog = TCPScanDialog(devices, get_location_strings, rescan,
@@ -699,7 +791,7 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
 
         try:
             devices = asphodel.find_tcp_devices()
-        except:
+        except Exception:
             devices = []
         chosen = self.choose_tcp_devices(devices)
         keys = self.get_tcp_device_keys(chosen)
@@ -845,22 +937,15 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
                 self.disconnected_tabs[widget] = reconnect_info
             self.should_scan.set()
 
-    def pause(self):
-        if not self.is_paused:
-            self.is_paused = True
-            for widget in self.tab_widgets:
-                widget.pause()
-
-    def resume(self):
-        if self.is_paused:
-            self.is_paused = False
-            for widget in self.tab_widgets:
-                widget.resume()
-
     def closeEvent(self, event):
+        geometry = self.saveGeometry()
+        self.settings.setValue('Geometry', geometry)
+
         self.finished.set()
         if self.upload_manager:
             self.upload_manager.stop()
+        if self.alert_manager:
+            self.alert_manager.stop()
 
         # notify widgets
         for widget in self.tab_widgets:
@@ -868,10 +953,12 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
 
         QtWidgets.QWidget.closeEvent(self, event)
 
-    def destroyed_cb(self, junk=None):
+    def destroyed_cb(self, _junk=None):
         self.finished.set()
         if self.upload_manager:
             self.upload_manager.stop()
+        if self.alert_manager:
+            self.alert_manager.stop()
 
     def update_widget_text(self, widget, text):
         try:
@@ -895,6 +982,8 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
                 self.should_scan.clear()
 
         try:
+            if self.shown_tab == widget:
+                self.shown_tab = None
             index = self.tab_widgets.index(widget)
             self.tabWidget.removeTab(index)
             self.tab_widgets.pop(index)
@@ -902,14 +991,18 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
         except ValueError:
             pass
 
+        if len(self.tab_widgets) == 0:
+            self.stackedWidget.setCurrentIndex(1)
+
     def show_tab(self, widget):  # called from a radio/remote panel
         self.tabWidget.setCurrentWidget(widget)
 
     def create_tab(self, proxy, sn, stream_list):  # called from a radio panel
         # create a new device tab
-        widget = DeviceTab(self.is_paused, sn, self.base_dir, self,
+        widget = DeviceTab(sn, self.base_dir, self.diskcache, self,
                            stream_list, self.disable_streaming,
-                           self.disable_archiving, self.upload_manager)
+                           self.disable_archiving, self.upload_manager,
+                           self.alert_manager, collapsed=self.collapsed)
 
         # find the insertion index
         index = len(self.tab_widgets)
@@ -921,49 +1014,12 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
         widget.name_update.connect(update_func)
         close_func = functools.partial(self.close_tab, widget)
         widget.close_pressed.connect(close_func)
+        widget.collapsed_set.connect(self.collapsed_set)
 
         widget.set_proxy(proxy)
 
         return widget
 
-    def recreate_tab(self, old_widget):
-        new_widget = DeviceTab(self.is_paused, old_widget.serial_number,
-                               self.base_dir, self, old_widget.stream_list,
-                               self.disable_streaming, self.disable_archiving,
-                               self.upload_manager)
-
-        serials_to_replace = [k for k, v in self.device_tabs.items() if
-                              v == old_widget]
-        for serial in serials_to_replace:
-            self.device_tabs[serial] = new_widget
-
-        with self.disconnected_tabs_lock:
-            if old_widget in self.disconnected_tabs:
-                reconnect_info = self.disconnected_tabs.pop(old_widget)
-                self.disconnected_tabs[new_widget] = reconnect_info
-
-        index = self.tab_widgets.index(old_widget)
-        self.tab_widgets[index] = new_widget
-
-        current_index = self.tabWidget.currentIndex()
-        text = self.tabWidget.tabText(index)
-        self.tabWidget.removeTab(index)
-        self.tabWidget.insertTab(index, new_widget, text)
-
-        if current_index == index:
-            self.tabWidget.setCurrentIndex(index)
-
-        update_func = functools.partial(self.update_widget_text, new_widget)
-        new_widget.name_update.connect(update_func)
-        close_func = functools.partial(self.close_tab, new_widget)
-        new_widget.close_pressed.connect(close_func)
-
-        if old_widget.proxy:
-            new_widget.set_proxy(old_widget.proxy)
-
-        # reconnect anything else
-        old_widget.recreate.emit(new_widget)
-
     def register_rf_power_panel(self, panel):
         def connect_callback(status):
             self.rf_power_panels[panel] = status
@@ -1012,34 +1068,67 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
         dialog = AboutDialog(self)
         dialog.exec_()
 
+    def get_htmL_dir(self):
+        is_frozen = getattr(sys, 'frozen', False)
+        if is_frozen:
+            # load the build_info.txt
+            return os.path.join(os.path.dirname(sys.executable), "html")
+        else:
+            return os.path.join(os.path.dirname(__file__), "html")
+
+    def show_channel_table(self):
+        filename = os.path.join(self.get_htmL_dir(), "asphodel_channels.html")
+        url = QtCore.QUrl.fromLocalFile(filename)
+        QtGui.QDesktopServices.openUrl(url)
+
+    def show_log_dir(self):
+        logdir = os.path.abspath(QtCore.QStandardPaths.writableLocation(
+            QtCore.QStandardPaths.DataLocation))
+        logfile = os.path.join(logdir, "main.log")
+
+        if sys.platform == "win32":
+            subprocess.Popen(['explorer', '/select,', logfile])
+        elif sys.platform == "darwin":
+            args = [
+                "osascript",
+                "-e", 'tell application "Finder"',
+                "-e", "activate",
+                "-e", f'select POSIX file "{logfile}"',
+                "-e", "end tell",
+            ]
+            subprocess.Popen(args)
+        else:
+            url = QtCore.QUrl.fromLocalFile(logdir)
+            QtGui.QDesktopServices.openUrl(url)
+
     def show_preferences(self):
         dialog = PreferencesDialog(self)
         if dialog.exec_():
             self.load_base_dir()
             self.create_upload_manager()
-            # updated base dir & upload manager requires restarting device tabs
+            self.create_alert_manager()
+
             for tab_widget in self.tab_widgets[:]:
                 if tab_widget in self.tab_widgets:
                     # hasn't been closed with another tab (like a WM)
-                    tab_widget.stop_for_recreate()
-                    self.recreate_tab(tab_widget)
+                    tab_widget.update_preferences(
+                        self.base_dir, self.upload_manager, self.alert_manager)
+        self.update_style()
 
-    def set_disable_streaming(self, checked=False):
+    def set_disable_streaming(self, _checked=False):
         self.disable_streaming = self.actionDisableStreaming.isChecked()
         for tab_widget in self.tab_widgets[:]:
             if tab_widget in self.tab_widgets:
                 # hasn't been closed with another tab (like a WM)
-                tab_widget.stop_for_recreate()
-                self.recreate_tab(tab_widget)
+                tab_widget.update_disable_streaming(self.disable_streaming)
 
-    def set_disable_archiving(self, checked=False):
+    def set_disable_archiving(self, _checked=False):
         self.disable_archiving = self.actionDisableArchiving.isChecked()
         self.warningLabel.setVisible(self.disable_archiving)
         for tab_widget in self.tab_widgets[:]:
             if tab_widget in self.tab_widgets:
                 # hasn't been closed with another tab (like a WM)
-                tab_widget.stop_for_recreate()
-                self.recreate_tab(tab_widget)
+                tab_widget.update_disable_archiving(self.disable_archiving)
 
     def update_datetime_label(self):
         s = datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ")
@@ -1130,3 +1219,29 @@ class PlotMainWindow(QtWidgets.QMainWindow, Ui_PlotMainWindow):
         if new_index < 0:
             new_index = self.tabWidget.count() - 1
         self.tabWidget.setCurrentIndex(new_index)
+
+    def collapsed_set(self, collapsed):
+        self.collapsed = collapsed
+        write_bool_setting(self.settings, "Collapsed", collapsed)
+        for tab_widget in self.tab_widgets:
+            tab_widget.set_collapsed(collapsed)
+
+    def update_style(self):
+        dark_mode = read_bool_setting(self.settings, "DarkMode", True)
+        set_style(QtWidgets.QApplication.instance(), dark_mode)
+
+    def current_tab_changed_cb(self, index):
+        if index == -1:
+            new_tab = None
+        else:
+            new_tab = self.tab_widgets[index]
+
+        if self.shown_tab == new_tab:
+            return  # no change
+
+        if self.shown_tab is not None:
+            self.shown_tab.set_is_shown(False)
+        if new_tab is not None:
+            new_tab.set_is_shown(True)
+
+        self.shown_tab = new_tab
