@@ -54,7 +54,7 @@ def bulk_claim(device):
                     remote.wait_for_connect(5000)
                     remote.set_device_mode(0)
                     device_logger.info("Claimed %s", serial_number)
-                except:
+                except Exception:
                     device_logger.error("Error Claiming %s", serial_number)
             device_logger.info("Finished Bulk Claim")
     finally:
@@ -197,7 +197,7 @@ def bulk_update(device, progress_pipe, cancel_pipe, firm_data, valid_range,
                     continue
 
                 reprogram_serials.add((app_mode, serial_number))
-            except:
+            except Exception:
                 device_logger.info("Error Querying {}".format(serial_number))
 
         if not reprogram_serials:
@@ -239,7 +239,7 @@ def bulk_update(device, progress_pipe, cancel_pipe, firm_data, valid_range,
                         # device mode
                         if device_mode is not None:
                             remote.set_device_mode(device_mode)
-                    except:
+                    except Exception:
                         tries -= 1
                         if tries == 0:
                             raise
@@ -248,7 +248,7 @@ def bulk_update(device, progress_pipe, cancel_pipe, firm_data, valid_range,
                     break
                 finished += 1
                 device_logger.info("Finished {}".format(serial_number))
-            except:
+            except Exception:
                 errors += 1
                 device_logger.exception("Error Programming {}".format(
                     serial_number))
@@ -269,18 +269,20 @@ def bulk_update(device, progress_pipe, cancel_pipe, firm_data, valid_range,
 
 @enum.unique
 class RadioState(enum.Enum):
-        UNINITIALIZED = 0
-        DISCONNECTED = 1
-        CONNECTING = 2
-        CONNECTED = 3
-        SCANNING = 4
+    UNINITIALIZED = 0
+    DISCONNECTED = 1
+    CONNECTING = 2
+    CONNECTED = 3
+    SCANNING = 4
 
 
 class RadioPanel(QtWidgets.QGroupBox, Ui_RadioPanel):
-    def __init__(self, start_usb_operation, parent=None):
+    def __init__(self, start_usb_operation, diskcache, parent=None):
         super().__init__(parent)
 
+        self.diskcache = diskcache
         self.device_tab = parent
+        self.logger = parent.logger
         self.start_usb_operation = start_usb_operation
 
         self.radio_scan_dialog = None
@@ -499,6 +501,9 @@ class RadioPanel(QtWidgets.QGroupBox, Ui_RadioPanel):
 
     def connected(self, device_info):
         # the device has been reconnected
+        if self.state != RadioState.UNINITIALIZED:
+            return  # wasn't disconnected, ignore
+
         self.devicesComboBox.clear()
         self.go_to_disconnected_state()
         self.has_scan_power = device_info['radio_scan_power']
@@ -655,6 +660,8 @@ class RadioPanel(QtWidgets.QGroupBox, Ui_RadioPanel):
             subproxy.close_connection()
             return
 
+        self.logger.info("Connected to {}".format(serial))
+
         if self.remote_tab:
             self.remote_tab.set_proxy(subproxy)
             self.go_to_connected_state()
@@ -671,16 +678,6 @@ class RadioPanel(QtWidgets.QGroupBox, Ui_RadioPanel):
             # reconnect when it disconnects
             self.remote_tab.disconnected_signal.connect(self.reconnect)
 
-            def recreate(new_tab):
-                self.remote_tab = new_tab
-                self.remote_tab.radio_tab = self.device_tab
-                self.remote_tab.close_pressed.connect(
-                    self.disconnect_button_cb)
-                self.remote_tab.recreate.connect(recreate)
-                self.remote_tab.disconnected_signal.connect(self.reconnect)
-
-            self.remote_tab.recreate.connect(recreate)
-
             self.go_to_connected_state()
             self.show_remote_tab()
 
@@ -693,6 +690,8 @@ class RadioPanel(QtWidgets.QGroupBox, Ui_RadioPanel):
 
     def close_remote_tab(self):
         if self.remote_tab:
+            self.logger.info("Closed connection to {}".format(
+                self.remote_tab.serial_number))
             self.device_tab.plotmain.close_tab(self.remote_tab)
             self.remote_tab = None
 
@@ -730,6 +729,8 @@ class RadioPanel(QtWidgets.QGroupBox, Ui_RadioPanel):
                                         self.last_connect_sn)
                 self.go_to_connecting_state()
                 if self.remote_tab:
+                    self.logger.info("Reconnecting to {}...".format(
+                        self.remote_tab.serial_number))
                     self.remote_tab.set_disconnected("Reconnecting...")
                     self.goToRemoteButton.setEnabled(True)
 
@@ -757,6 +758,7 @@ class RadioPanel(QtWidgets.QGroupBox, Ui_RadioPanel):
 
         # start the background thread
         self.start_usb_operation(self.start_detail_scan_op,
+                                 self.diskcache,
                                  self.tx_status_pipe,
                                  self.rx_control_pipe)
 
@@ -819,7 +821,7 @@ class RadioPanel(QtWidgets.QGroupBox, Ui_RadioPanel):
 
         try:
             firm_data = bootloader.decode_firm_file(firm_file)
-        except:
+        except Exception:
             self.logger.exception('Error loading firmware from "%s"',
                                   firm_file)
             m = self.tr('Error loading firmware from file!').format(firm_file)
