@@ -9,6 +9,7 @@ from PySide2 import QtCore, QtGui, QtWidgets
 
 import asphodel
 from hyperborea.preferences import read_bool_setting
+import hyperborea.device_info
 import hyperborea.proxy
 
 from .ui_radio_scan_dialog import Ui_RadioScanDialog
@@ -55,10 +56,11 @@ def stop_radio_scan_manager(device):
 
 
 class RadioScanManager:
-    def __init__(self, device, logger, status_pipe, control_pipe):
+    def __init__(self, device, logger, diskcache, status_pipe, control_pipe):
         self.device = device
         self.remote = device.get_remote_device()
         self.logger = logger
+        self.diskcache = diskcache
         self.status_pipe = status_pipe
         self.control_pipe = control_pipe
 
@@ -96,26 +98,21 @@ class RadioScanManager:
 
             self.remote.wait_for_connect(1000)
 
-            tag_locations = self.remote.get_user_tag_locations()
-            try:
-                t1 = self.remote.read_user_tag_string(*tag_locations[0])
-            except UnicodeDecodeError:
-                t1 = None
-            try:
-                t2 = self.remote.read_user_tag_string(*tag_locations[1])
-            except UnicodeDecodeError:
-                t2 = None
-            board_info = self.remote.get_board_info()
-            build_info = self.remote.get_build_info()
-            build_date = self.remote.get_build_date()
-            has_bootloader = bool(self.remote.get_bootloader_info())
+            device_info = hyperborea.device_info.get_active_scan_info(
+                self.device, self.remote, self.diskcache)
+
+            t1 = device_info['user_tag_1']
+            t2 = device_info['user_tag_2']
+            board_info = device_info['board_info']
+            build_info = device_info['build_info']
+            build_date = device_info['build_date']
+            has_bootloader = bool(device_info['bootloader_info'])
 
             last_seen = datetime.datetime.now(datetime.timezone.utc)
             s = ScanResult(True, sn, is_boot, None, None, last_seen, t1, t2,
                            board_info, build_info, build_date, has_bootloader)
             self.status_pipe.send(s)
-
-        except:
+        except Exception:
             # couldn't do the active scan
             # no big deal
             pass
@@ -225,8 +222,7 @@ class RadioScanManager:
                         self.collect_scan_results()
                 restart_scan = True
                 scan_stop = time.time() + 0.6
-
-        except:
+        except Exception:
             self.logger.exception("Unhandled exception in scan_thread_run")
             self.status_pipe.send("error")
 
@@ -262,27 +258,36 @@ class RadioScanTableModel(QtCore.QAbstractTableModel):
         scan_result = self.scan_list[index.row()]
         col = index.column()
 
-        if role == QtCore.Qt.BackgroundRole:
-            if scan_result.is_bootloader:
-                return QtGui.QColor(QtCore.Qt.yellow)
-            elif col == 9:
+        if role != QtCore.Qt.DisplayRole:
+            if col == 9:
                 now = datetime.datetime.now(datetime.timezone.utc)
                 delta = now - scan_result.last_seen
                 seconds = int(delta.total_seconds())
                 if seconds > 30:
-                    return QtGui.QColor(QtCore.Qt.red)
+                    if role == QtCore.Qt.BackgroundRole:
+                        return QtGui.QColor(QtCore.Qt.red)
+                    elif role == QtCore.Qt.ForegroundRole:
+                        return QtGui.QColor(QtCore.Qt.black)
                 elif seconds > 10:
+                    if role == QtCore.Qt.BackgroundRole:
+                        return QtGui.QColor(QtCore.Qt.yellow)
+                    elif role == QtCore.Qt.ForegroundRole:
+                        return QtGui.QColor(QtCore.Qt.black)
+
+            if scan_result.is_bootloader:
+                if role == QtCore.Qt.BackgroundRole:
                     return QtGui.QColor(QtCore.Qt.yellow)
-            return None
-        if role == QtCore.Qt.FontRole:
-            if col == 7:
-                if scan_result.is_bootloader:
+                elif role == QtCore.Qt.ForegroundRole:
+                    return QtGui.QColor(QtCore.Qt.black)
+                elif role == QtCore.Qt.FontRole and col == 7:
                     font = QtGui.QFont()
                     font.setBold(True)
                     return font
+
+            # default for non-display roles
             return None
-        elif role != QtCore.Qt.DisplayRole:
-            return None
+
+        # must be a display role (i.e. return the string contents)
 
         if col == 0:  # Serial Number
             return str(scan_result.serial_number)
@@ -580,7 +585,7 @@ class RadioScanDialog(QtWidgets.QDialog, Ui_RadioScanDialog):
         if filter_level is not None:
             try:
                 filter_level = int(filter_level)
-            except:
+            except ValueError:
                 filter_level = self.filterSlider.minimum()  # default
         else:
             filter_level = self.filterSlider.minimum()  # default
